@@ -1,14 +1,13 @@
 # -*- coding: utf-8 -*-
 import logging
-
+from boto3.session import Session
 from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type
 
 from .converter import Converter
 from .cursor import BaseCursor, Cursor
 from .error import NotSupportedError
-from .util import RetryConfig
+from .util import RetryConfig, retry_api_call
 
-from boto3.session import Session
 
 if TYPE_CHECKING:
     from botocore.client import BaseClient
@@ -74,6 +73,7 @@ class Connection:
         self.cursor_kwargs = cursor_kwargs if cursor_kwargs else dict()
         self._cursor_pool = list()
         self._autocommit = True
+        self._in_transaction = False
 
     @property
     def _session_kwargs(self) -> Dict[str, Any]:
@@ -97,6 +97,24 @@ class Connection:
     def autocommit(self) -> bool:
         return self._autocommit
 
+    @autocommit.setter
+    def autocommit(self, value: bool) -> None:
+        try:
+            if not self._autocommit and value:
+                self._autocommit = True
+                for cursor_ in self.cursor_pool:
+                    cursor_.flush()
+        finally:
+            self._autocommit = value
+
+    @property
+    def in_transaction(self) -> bool:
+        return self._in_transaction
+
+    @in_transaction.setter
+    def in_transaction(self, value: bool) -> bool:
+        self._in_transaction = False
+
     @property
     def cursor_pool(self) -> List[Optional[BaseCursor]]:
         return self._cursor_pool
@@ -128,15 +146,37 @@ class Connection:
         self._session = None
         self._client = None
         self._autocommit = True
+        self._in_transaction = False
         self.cursor_pool.clear()
 
     def begin(self) -> None:
         self._autocommit = False
+        self._in_transaction = True
 
     def commit(self) -> None:
-        self._autocommit = True
-        for cursor_ in self.cursor_pool:
-            cursor_.execute_transaction()
+        try:
+            if self._in_transaction:
+                for cursor_ in self.cursor_pool:
+                    cursor_.execute_transaction()
+        finally:
+            self._autocommit = True
+            self._in_transaction = False
 
     def rollback(self) -> None:
         raise NotSupportedError
+
+    def test_connection(self) -> bool:
+        request: Dict[str, Any] = {"Limit": 1}
+
+        try:
+            retry_api_call(
+                self._client.list_tables,
+                config=self._retry_config,
+                logger=_logger,
+                **request,
+            )
+        except Exception as e:
+            _logger.exception("Failed to connect database: %s" % repr(e))
+            return False
+        else:
+            return True
